
/*
Eagles Bulletin Board System
Copyright (C) 1994, Ray Rocker, rrrocker@rock.b11.ingr.com
                                rock@seabass.st.usm.edu
                                72673.2105@compuserve.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "client.h"
#include <stdlib.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>

int
OpenBoard(openflags, newonly, resp)
int *openflags;
int newonly;
int *resp;
{
  int code;
  OPENINFO openinfo;
  code = bbs_open_board(currboard, &openinfo);    
  if (resp) *resp = code;
  if (code != S_OK) return -1;
  if (openflags) *openflags = openinfo.flags;
  return (newonly ? openinfo.newmsgs : openinfo.totalmsgs);
}

CloseBoard()
{
  int code;
  code = bbs_close_board();
  return (code == S_OK ? 0 : -1);
}

/* 
   Return codes for DoPostSend:
   0 == success
   -1 == posting failed
   -2 == post was manually aborted
   -3 == post is zero length -- aborted
*/

DoPostSend(subject, textfile, doedit)
char *subject;    /* subject string */
char *textfile;   /* file to send: if NULL invoke editor */
int doedit;       /* edit even if textfile != NULL */
{
  int rc = 0;
  struct stat stbuf;
  
  if (textfile == NULL || doedit) {
    if (textfile == NULL) textfile = c_tempfile;
    if (Edit(textfile)) { 
      if (textfile == c_tempfile) unlink(c_tempfile);
      return -2;
    }
  }
  if (stat(textfile, &stbuf) != 0 || stbuf.st_size == 0) {
    rc = -3;
  } else {
    OPENINFO openinfo;
    if (bbs_open_board(currboard, &openinfo) != S_OK) {
      rc = -1;
    }
    if (rc == 0 && bbs_post(currboard, subject, textfile) != S_OK) {
      rc = -1;
    }
    bbs_close_board();
  }
  if (textfile == c_tempfile) unlink(c_tempfile);
  return rc;
}

FtpPost ()
{
  int rc;
  LONG retcode = FULLUPDATE;
  TITLE subject;
  SHORT flags;
  char ftphost[64];
  char uname[12];
  char passwd[32];
  char fname[128];
  char arg[256];
  FILE *in, *out;
  int c;
  extern int lang;
  extern FILE *FtpFullOpen();
  ACCOUNT acct;  /*  */

  clear();
  if (currboard[0] == '\0') {
    c_msg(ERR_NOT_SELECTED, TRUE);
    pressreturn();
    return FULLUPDATE;
  }
  if (bbs_test_board(currboard, &flags) != S_OK) {
    c_msg(ERR_NOT_EXIST, TRUE);
    pressreturn();
    return FULLUPDATE;
  }
  if (!BITISSET(flags, OPEN_POST)) {
    c_msg(ERR_PERM, TRUE);
    pressreturn();
    return FULLUPDATE;
  }
  bbs_set_mode(M_FTP);
  c_msg(INFO_POSTING, TRUE);
  if (lang) {
    getdata(1, 0, ": ", subject, sizeof(subject), DOECHO, 0);
  } else {
    getdata(1, 0, "Subject: ", subject, sizeof(subject), DOECHO, 0);
  }

  clear();

  memset(ftphost, '\0', 64);
  memset(uname, '\0', 12);
  memset(passwd, '\0', 32);
  memset(fname, '\0', 128);
  memset(arg, '\0', 256);

 move(1,0);
 c_msg(INFO_FTP_POSTING, TRUE);
 if (lang) {
  if (getdata(2, 0, " FTP?  : ", ftphost, sizeof(ftphost), DOECHO, 1) == -1 ||
      getdata(3, 0, "?      : ", uname, sizeof(uname), DOECHO, 1) == -1 ||
      getdata_echo(4, 0, "ȣ?      : ", passwd, sizeof(passwd), NOECHO, '*', 1) == -1 ||
      getdata(5, 0, " ȭ? : ", fname, sizeof(fname), DOECHO, 1) == -1)
    {
      c_msg(INFO_CANCELLED, TRUE);
      pressreturn();
      bbs_set_mode(M_UNDEFINED);
      return FULLUPDATE;
    }
 } else {
  if (getdata(2, 0, "FTP from?  : ", ftphost, sizeof(ftphost), DOECHO, 1) == -1 ||
      getdata(3, 0, "FTP user?  : ", uname, sizeof(uname), DOECHO, 1) == -1 ||
      getdata_echo(4, 0, "FTP passwd?: ", passwd, sizeof(passwd), NOECHO, '*', 1) == -1 ||
      getdata(5, 0, "File Name? : ", fname, sizeof(fname), DOECHO, 1) == -1)
    {
      c_msg(INFO_CANCELLED, TRUE);
      pressreturn();
      bbs_set_mode(M_UNDEFINED);
      return FULLUPDATE;
    }
 }

  sprintf(arg, "%s/%s@%s:%s", uname, passwd, ftphost, fname);

  if ((in = (FILE*)Ftpfopen(arg, "rb")) == NULL) {
    c_msg(ERR_FAILURE, TRUE);
    pressreturn();
    bbs_set_mode(M_UNDEFINED);
    return FULLUPDATE;
  }

  if ((out = fopen(c_tempfile, "w")) == NULL) {
    c_msg(ERR_FAILURE, TRUE);
    pressreturn();
    bbs_set_mode(M_UNDEFINED);
    return FULLUPDATE;
  }
  while ((c=getc(in)) != EOF)
    putc(c, out);
  Ftpfclose(in);
  fclose(out);

  clear();

  rc = DoPostSend(subject, c_tempfile, 0);

  clear();
  move(0,0);
  switch (rc) {
  case -3:
    c_msg(ERR_ZERO, TRUE);
    break;
  case -2:
    c_msg(ERR_FAILURE, TRUE);
    break;
  case -1:
    c_msg(ERR_FILE, TRUE);
    break;
  case 0:
    c_msg(INFO_POSTED, TRUE);
    retcode |= FETCHNEW;
    bbs_find_userinfo(myinfo.userid, &acct);  /*  */
    local_bbs_set_loginNposts(0, acct);  /*  */
    break;
  }
  pressreturn();

  /* If docheck is true,  we came in from the main menu -- else, read menu */
  bbs_set_mode(M_UNDEFINED);
  return retcode;
}

GenericPost(docheck)
int docheck;
{
  ACCOUNT acct;  /*  */
  int rc;
  LONG retcode = FULLUPDATE;
  TITLE subject;
  extern int lang;
  
  clear();
  if (docheck) {
    SHORT flags;
    if (currboard[0] == '\0') {
      c_msg(ERR_NOT_SELECTED, TRUE);
      pressreturn();
      return FULLUPDATE;
    }
    if (bbs_test_board(currboard, &flags) != S_OK) {
      c_msg(ERR_NOT_EXIST, TRUE);
      pressreturn();
      return FULLUPDATE;
    }
    if (!BITISSET(flags, OPEN_POST)) {
      c_msg(ERR_PERM, TRUE);
      pressreturn();
      return FULLUPDATE;
    }
  }
  bbs_set_mode(M_POSTING);
  c_msg(INFO_POSTING, TRUE);
  if (lang) {
    getdata(1, 0, ": ", subject, sizeof(subject), DOECHO, 0);
  } else {
    getdata(1, 0, "Subject: ", subject, sizeof(subject), DOECHO, 0);
  }
  rc = DoPostSend(subject, (char *)NULL, 0);
  clear();
  move(0,0);
  switch (rc) {
  case -3:
    c_msg(ERR_ZERO, TRUE);
    break;
  case -2:
    c_msg(ERR_FAILURE, TRUE);
    break;
  case -1:
    c_msg(ERR_FILE, TRUE);
    break;
  case 0:
    c_msg(INFO_POSTED, TRUE);
    retcode |= FETCHNEW;
    bbs_find_userinfo(myinfo.userid, &acct);   /*  */
    local_bbs_set_loginNposts(0, acct);  /*  */
    break;
  }
  pressreturn();
  /* If docheck is true,  we came in from the main menu -- else, read menu */
  bbs_set_mode(docheck ? M_UNDEFINED : M_READING);
  return retcode;
}

Post()
{
  int rc;
  rc = GenericPost(1);
  return (rc);
}

/*ARGSUSED*/
PostMessage(hptr, currmsg, numrecs, openflags)
HEADER *hptr;
int currmsg, numrecs, openflags;
{
  int rc;
  if (!BITISSET(openflags, OPEN_POST)) {
    bell();
    return DONOTHING;
  }
  rc = GenericPost(0);   /* no need to check post privilege -- we do */
  return (rc);
}

/*ARGSUSED*/
ReadMenuSelect(hptr, currmsg, numrecs, openflags)
HEADER *hptr;
int currmsg, numrecs, openflags;
{
  return (SelectBoard());
}

GenericPostReply(hptr, msgsrc)
HEADER *hptr;
char *msgsrc;
{
  TITLE subject;
  TITLE tmp_title; /*  */
  char ans[4];
  int rc, retcode = FULLUPDATE;
  PATH msgfile;
  extern int lang;
  ACCOUNT acct;  /*  */


  clear();

  if (msgsrc == NULL) {
    if (bbs_read_message(hptr->fileid, msgfile) != S_OK) {
      c_msg(ERR_FILE, TRUE);
      pressreturn();
      return retcode;
    }
  }
  else strcpy(msgfile, msgsrc);

/*  */
  move(0,0);
  prints("[");
  standout(); 
  prints(" %s ", currboard); 
  standend(); 
  if (lang) prints("] 忡 ︮  ּ.\n");
  else prints("] is current board.\n");
/*  */

  move(2,0);
  bbs_set_mode(M_POSTING);
  c_msg(INFO_REPLY, TRUE);
  if (lang)
    getdata(1, 0, ": ", subject, sizeof(subject), DOECHO, 0);
  else
    getdata(1, 0, "Title: ", subject, sizeof(subject), DOECHO, 0);

  if (*subject == '\0') {
    if (!strncasecmp(hptr->title, "Re:", 3)) {
      strncpy(subject, hptr->title, TITLELEN-1);
    }
    else if (!strncasecmp(hptr->title, "[Q]", 3)) {
      hptr->title[1]='A';
      strncpy(subject, hptr->title, TITLELEN-1);
    }
    else if (!strncasecmp(hptr->title, "[A]", 3)) { 
      strncpy(subject, hptr->title, TITLELEN-1);
    }
    else if (!strncasecmp(hptr->title, "[I]", 3)) {
      strncpy(subject, hptr->title, TITLELEN-1);
    }    
    else {
      strcpy(subject, "Re: ");
      strncat(subject, hptr->title, TITLELEN-5);
    }
    move(1,0);
    if (lang)
      prints(": %s\n", subject);
    else
      prints("Title: %s\n", subject);
  }    

  if (*subject == '/') {
    if (!strncasecmp(hptr->title, "Re:", 3)) {
      strncpy(tmp_title, hptr->title, TITLELEN-1-strlen(subject));
      strncat(tmp_title, subject, strlen(subject));
    }
    else if (!strncasecmp(hptr->title, "[Q]", 3)) {
      hptr->title[1]='A';
      strncpy(tmp_title, hptr->title, TITLELEN-1-strlen(subject));
      strncat(tmp_title, subject, strlen(subject));
    }    
    else if (!strncasecmp(hptr->title, "[A]", 3)) { 
      strncpy(tmp_title, hptr->title, TITLELEN-1-strlen(subject)); 
      strncat(tmp_title, subject, strlen(subject));
    }    
    else if (!strncasecmp(hptr->title, "[I]", 3)) {  
      strncpy(tmp_title, hptr->title, TITLELEN-1-strlen(subject));  
      strncat(tmp_title, subject, strlen(subject));
    }
    else {
      strcpy(tmp_title, "Re: "); 
      strncat(tmp_title, hptr->title, TITLELEN-5-strlen(subject));
      strncat(tmp_title, subject, strlen(subject));
    }
    strncpy(subject, tmp_title, TITLELEN-1);
    move(1,0);
    if (lang)
      prints(": %s\n", subject);
    else
      prints("Title: %s\n", subject);
  } /* а ߰ /id1/id2...  */ 

  if (lang)
    getdata(2, 0, " ÿ Խų (Y/N)? [N]: ",
	  ans, sizeof(ans), DOECHO, 0);
  else
    getdata(2, 0, "Include message text in followup (Y/N)? [N]: ",
	  ans, sizeof(ans), DOECHO, 0);
  if (*ans == 'Y' || *ans == 'y') {
    AnnotateMessageBody(c_tempfile, msgfile);
  }

  rc = DoPostSend(subject, c_tempfile, 1);
  clear();
  move(0,0);
  switch (rc) {
  case -3:
    c_msg(ERR_ZERO, TRUE);
    break;
  case -2:        
    c_msg(ERR_FAILURE, TRUE);
    break;
  case -1:
    c_msg(ERR_FILE, TRUE);
    break;
  case 0:
    c_msg(INFO_POSTED, TRUE);
    retcode |= FETCHNEW;
/*  */
    bbs_find_userinfo(myinfo.userid, &acct);
    /* printf("\n ... ̵=%-12s\n",acct.userid); */
    local_bbs_set_loginNposts(0, acct); 
/*  */
    break;
  }
  pressreturn();
  bbs_set_mode(M_READING);
  return retcode;
}    

/*ARGSUSED*/
PostReply(hptr, currmsg, numrecs, openflags)
HEADER *hptr;
LONG currmsg, numrecs, openflags;
{
  return (GenericPostReply(hptr, NULL));
}

/*ARGSUSED*/
PostDelete(hptr, currmsg, numrecs, openflags)
HEADER *hptr;
int currmsg, numrecs, openflags;
{
  int rc = FULLUPDATE;
  char ans[4];
  extern int lang;

  if (strcmp(myinfo.userid,hptr->owner) && !BITISSET(openflags,OPEN_MANAGE)) {
    bell();
    return DONOTHING;
  }
  clear();
  c_msg(INFO_DELETING, TRUE);
  if (lang)
  {
    getdata(2,0,"  (Y/N)? [N]: ",ans,sizeof(ans),DOECHO, 0);
  }
  else
  {
    getdata(2,0,"Are you sure (Y/N)? [N]: ",ans,sizeof(ans),DOECHO, 0);
  }
  move(4,0);
  if (*ans != 'Y' && *ans != 'y') {
    c_msg(INFO_CANCELLED, TRUE);
  } else {
    if (bbs_delete_message(hptr->fileid) == S_OK) {
      c_msg(INFO_DELETED, TRUE);
      rc |= FETCHNEW;
    } else {
      c_msg(ERR_FAILURE, TRUE);
    }
  }
  bbslog(2, "DELETEPOST %d by %s\n", hptr->fileid, myinfo.userid);
  pressreturn();
  return rc;
}

/*ARGSUSED*/
PostDelRange(hptr, currmsg, numinbox, openflags)
HEADER *hptr;
int currmsg, numinbox, openflags;
{
  int rc = FULLUPDATE;
  char ans[5];
  SHORT n1 = 0, n2 = 0, deleted;
  if (!BITISSET(openflags, OPEN_MANAGE)) {
    bell();
    return DONOTHING;
  }
  clear();
  if (numinbox <= 0) {
    c_msg(ERR_NOTHING, TRUE);
    pressreturn();
    return rc;
  }
  c_msg(INFO_DELETE_RANGE, TRUE);
  do {
    if (getdata(2,0,"Enter number of first message to delete: ",
	    ans,sizeof(ans),DOECHO,1) == -1) return rc;
    n1 = atoi(ans);
    if (n1 <= 0 || n1 > numinbox) {
      bell();
      move(3,0);
      c_msg(ERR_RANGE, TRUE);
    }        
  } while (n1 <= 0 || n1 > numinbox);
  do {
    if (getdata(3,0,"Enter number of last message to delete: ",
	    ans,sizeof(ans),DOECHO,1) == -1) return rc;
    n2 = atoi(ans);
    if (n2 < n1 || n2 > numinbox) {
      bell();
      move(4,0);
      c_msg(ERR_RANGE, TRUE);
    }        
  } while (n2 < n1 || n2 > numinbox);
  
  move(4,0);
  clrtobot();
  getdata(4,0,"Are you sure (Y/N)? [N]: ",ans,sizeof(ans),DOECHO,0);
  move(6,0);
  if (*ans != 'Y' && *ans != 'y') {
    c_msg(INFO_CANCELLED, TRUE);
  } else {
    if (bbs_delete_range(n1, n2, &deleted) == S_OK) {
      c_msg(INFO_DELETED, TRUE);
      rc |= FETCHNEW;
    } else {
      c_msg(ERR_FAILURE, TRUE);
    }
  }
  pressreturn();
  return rc;
}

/*ARGSUSED*/
PostMark(hptr, currmsg, numrecs, openflags)
HEADER *hptr;
int currmsg, numrecs, openflags;
{
  SHORT ismarked = BITISSET(hptr->flags, FILE_MARKED);
  if (bbs_mark_message(hptr->fileid, !ismarked) != S_OK) return DONOTHING;
  if (ismarked) BITCLR(hptr->flags, FILE_MARKED);
  else BITSET(hptr->flags, FILE_MARKED);
  return PARTUPDATE;
}            

#define	UNMARKED_FILE_MODE	0660

/*ARGSUSED*/
PostEdit(hptr, currmsg, numrecs, openflags)
HEADER *hptr;
int currmsg, numrecs, openflags;
{
  struct stat stbuf;
  PATH msgfile;
  int fd;

  if (strcmp(myinfo.userid,hptr->owner) && !BITISSET(openflags,OPEN_MANAGE)) {
    bell();
    return DONOTHING;
  }
  if (bbs_read_message(hptr->fileid, msgfile) != S_OK) {
    c_msg(ERR_FILE, TRUE);
    pressreturn();
    return FULLUPDATE;
  }
  CopyMessageBody(c_tempfile, msgfile);
  if (Edit(c_tempfile)) { 
    c_msg(ERR_EDIT, TRUE);
    pressreturn();
    return FULLUPDATE;
  }
  if (stat(c_tempfile, &stbuf) != 0 || stbuf.st_size == 0) {
    unlink(c_tempfile);
    c_msg(ERR_ZERO, TRUE);
    pressreturn();
    return FULLUPDATE;
  }
  unlink (msgfile);
  fd = open(msgfile,  O_WRONLY|O_CREAT|O_EXCL, UNMARKED_FILE_MODE);
  if (fd == -1) {
    unlink(c_tempfile);
    c_msg(ERR_FILE, TRUE);
    pressreturn();
    return FULLUPDATE;
  }
  (void) write_post_headers(fd, hptr, myinfo.username, currboard);
  if (append_file (fd,  c_tempfile) == -1) {
    close(fd);
    unlink(c_tempfile);
    c_msg(ERR_FILE, TRUE);
    pressreturn();
    return FULLUPDATE;
  }
  close(fd);

  if (bbs_update_message(hptr->fileid, msgfile) == S_OK) {
    c_msg(INFO_EDITTED, TRUE);
  } else {
    c_msg(ERR_EDIT, TRUE);
  }
  return FULLUPDATE;
}            

/*ARGSUSED*/
PostDisplay(hptr, currmsg, numrecs, openflags)
HEADER *hptr;
int currmsg, numrecs, openflags;
{
  extern int lang;
  char ans[2], promptstr[80];
  PATH msgfile;
  int rc = FULLUPDATE;
  int done = 0, replied = 0, marked = 0;
#if !NO_POST_FOLLOWUP
  int posted = 0;
#endif
#if REMOTE_CLIENT
  int saved = 0;
#endif
  if (bbs_read_message(hptr->fileid, msgfile) != S_OK) {
    c_msg(ERR_FILE, TRUE);
    pressreturn();
    return rc;
  }
  BITCLR(hptr->flags, FILE_UNREAD);
  clear();
  More(msgfile, 0);
  while (!done) {
    promptstr[0] = '\0';
    if (!replied && HasPerm(C_MAIL)) 
    {
      if (lang)
        strcat(promptstr, "[M]- , ");
      else
        strcat(promptstr, "Mail reply, ");
    }
#if !NO_POST_FOLLOWUP
    if (!posted && BITISSET(openflags, OPEN_POST))
    {
      if (lang)
        strcat(promptstr, "[P]-ø, ");
      else
        strcat(promptstr, "Post followup, ");
    }
#endif
#if REMOTE_CLIENT
    if (!saved) strcat(promptstr, "Save, ");
#endif

    if (!marked && BITISSET(openflags, OPEN_MANAGE))
      strcat(promptstr, 
	     BITISSET(hptr->flags, FILE_MARKED) ? "unmarK, " : "marK, ");

    if (BITISSET(openflags, OPEN_MANAGE) || 
             !strcmp(hptr->owner, myinfo.userid))
    {
      if (lang)
        strcat(promptstr, "[D]-, ");
      else
        strcat(promptstr, "Delete, ");
    }

    if (promptstr[0] == '\0') {
      pressreturn();
      break;
    }
    else
    {
      if (lang)
        strcat(promptstr, "[C]-? [C]: ");
      else
        strcat(promptstr, "or Continue? [C]: ");
    }
/*    getdata(t_lines-1, 0, promptstr, ans, sizeof(ans), DOECHO, 0); */
    getdata(t_lines-1, 0, promptstr, ans, 1, DOECHO, 0); /*(̽)*/
    switch (*ans) {
    case 'm': case 'M': 
      if (replied) done = 1;
      else {
	rc |= MailReply(hptr, currmsg, numrecs, openflags);
	replied = 1;
      }
      break;
#if !NO_POST_FOLLOWUP
    case 'p': case 'P':
      if (posted) done = 1;
      else {
	rc |= GenericPostReply(hptr, msgfile);
	posted = 1;
      }
      break;
#endif
#if REMOTE_CLIENT
    case 's': case 'S':
      if (saved) done = 1;
      else {
	SaveFile(msgfile);
	saved = 1;
      }
      break;
#endif
    case 'k': case 'K': 
      if (marked) done = 1;
      else {
	PostMark(hptr, currmsg, numrecs, openflags);
	marked = 1;
      }
      break;
    case 'd': case 'D': 
      if (marked) done = 1;
      else rc |= PostDelete(hptr, currmsg, numrecs, openflags);
      /* fall through */
    default:
      done = 1;
      break;
    }
  }
  return rc;
}
