
/*
Eagles Bulletin Board System
Copyright (C) 1994, Ray Rocker, rrrocker@rock.b11.ingr.com
                                rock@seabass.st.usm.edu
                                72673.2105@compuserve.com

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "client.h"
#include <string.h>
#include <fcntl.h>
#include <time.h>

#if REMOTE_CLIENT
extern char *editor;
extern char *pager;
extern char *shell;
#endif

static char *day_of_week[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
static char *mon_of_year[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};

ShowDate()
{
  time_t now;
  extern int lang;
  extern char *Ctime __P((time_t *));

  move(2,0);
  clrtobot();
  move(3,0);

  time(&now);

  if (lang)
    prints(" ð: %s", Ctime(&now));
  else
    prints("Date and time on this system: %s", Ctime(&now));
  return PARTUPDATE;
}

/* 
   The builtin more should be improved a bit. I decided to leave it
   non-configurable because (a) no one has complained loudly, and (b)
   why bother with all that forking and execing when reading posts.
*/

More(filename, promptend)
char *filename;
int promptend; 
{
  return (more(filename, promptend));
}

#if !REMOTE_CLIENT
SelectEditor()
{
  NAME editorname;
  NAMELIST editorlist = NULL;
  extern LOGININFO myinfo;
  extern int lang;

  move(2,0);
  clrtobot();
  bbs_enum_editors(&editorlist);
  if (editorlist == NULL) {
    c_msg(ERR_NOTHING, TRUE);
    pressreturn();
    return FULLUPDATE;
  }

  if (lang)
    namecomplete(editorlist, "⸦ : ", editorname);
  else
    namecomplete(editorlist, "Select editor: ", editorname);

  if (editorname[0] == '\0') return FULLUPDATE;
  else if (!is_in_namelist(editorlist, editorname)) {
    move(4,0);
    c_msg(ERR_NOT_EXIST, TRUE);
    free_namelist(&editorlist);
    pressreturn();
    return FULLUPDATE;
  }
  if (*editorname)
    strcpy(myinfo.editor, editorname);
  else
    strcpy(myinfo.editor, "builtin");
  bbs_set_editor(editorname);
  free_namelist(&editorlist);
  return FULLUPDATE;
}
#endif

Edit(filename)
char *filename;
{
  int rc;
  int done = 0;
#if !REMOTE_CLIENT
  ACCOUNT acct;
  PATH editbin;
  PATH editorenv;
  PATH execbuf;
  char ans[2];

  acct.editor[0] = '\0';
  bbs_owninfo(&acct);
  if (acct.editor[0] != '\0') {
    if (bbs_get_editor(acct.editor, editbin, editorenv) == S_OK) {
      extern int lang;

      sprintf(editorenv, "TERM=%s", acct.terminal);
      if (editorenv[0] != '\0') parse_environment(editorenv);
      if (strcmp(acct.editor, "PICO") == 0 || strcmp(acct.editor, "pico") == 0) {
	if (lang)
          sprintf(execbuf, "%s -t -k %s", editbin, filename);
	else
          sprintf(execbuf, "%s -t %s", editbin, filename);
      } else {
        if (strcmp(acct.editor, "VI") == 0 || strcmp(acct.editor, "vi") == 0) {
          sprintf(execbuf, "%s -s %s", editbin, filename);
	} else {
          sprintf(execbuf, "%s %s", editbin, filename);
	}
      }
      while (!done) {
	bbs_set_mode(1);    /* Magic value to block paging */
	rc = do_exec(execbuf, NULL, 0);
	bbs_set_mode(0);    /* Magic value to allow paging */
	clear();
	if (rc) {
	  c_msg(ERR_EXEC, TRUE);
	  unlink (filename);
	  pressreturn();
	  return (rc);
	}
	if (lang)
	  getdata(0,0,"(S), (A), (E)? [S]: ",ans,sizeof(ans),DOECHO,0);
	else
	  getdata(0,0,"(S)ave, (A)bort, (E)dit? [S]: ",ans,sizeof(ans),DOECHO,0);
	switch (ans[0]) {
	case 'E': case 'e':
	  break;
	case 'A': case 'a':
	  unlink(filename);
	  rc = 1;
	  done = 1;
	  break;
	default:
	  rc = 0;
	  done = 1;
	  break;
	}
      }
      if (lang)
        getdata(0,0," ? (Y/N)? [Y]: ",ans, 2, DOECHO, 0);
      else
        getdata(0,0,"Post it? (Y/N)? [Y]: ",ans, 2, DOECHO, 0);
      if (*ans == 'N' || *ans == 'n')
	return (-1);
      return (0);
    } else {
      clear();
      c_msg(ERR_NOT_EXIST, TRUE);
      pressreturn();
    }
  }
  rc = vedit(filename);
  return (rc);
#else /* REMOTE_CLIENT */
  char buf[MAXCOMSZ];
  char ans[9];
  if (editor == NULL) {
    return(vedit(filename));
  }
  sprintf(buf, "%s %s", editor, filename);
  while (!done) {    
    rc = do_exec(buf, NULL, 0);
    clear();
    if (rc == -1) {
      c_msg(ERR_EXEC, TRUE);
      pressreturn();
      break;
    }
    if (lang)
      getdata(0,0,"(C)-, (A)-, (E)-? [C]: ",ans,sizeof(ans),DOECHO,0);
    else
      getdata(0,0,"(C)-Continue, (A)-Abort, (E)-Edit? [C]: ",ans,sizeof(ans),DOECHO,0);
    switch (ans[0]) {
    case 'E': case 'e':
      break;
    case 'A': case 'a':
      unlink(filename);
      rc = done = 1;
      break;
    default:
      rc = 0;
      done = 1;
    }
  }
  return rc; 
#endif
}
 
#if REMOTE_CLIENT
ShellEscape()
{
  clear();
  do_exec(shell, NULL, 0);
  return FULLUPDATE;
}
#endif

Welcome()
{
  PATH welcfile;
  clear();
  if (bbs_get_welcome(welcfile) != S_OK || More(welcfile, 1)) {
    c_msg(ERR_FAILURE, TRUE);
    pressreturn();
  }
  return FULLUPDATE;
}
    
EditWelcome()
{
  PATH welcfile;
  if (bbs_get_welcome(welcfile) != S_OK) {
    c_msg(ERR_FAILURE, TRUE);
    pressreturn();
    return FULLUPDATE;
  }
  switch (Edit(welcfile)) {
  case -1:
    break;
  case 1:
    move(1,0);
    c_msg(ERR_FAILURE, TRUE);
    pressreturn();
    break;        
  case 0:
    move(1,0);
    if (bbs_set_welcome(welcfile) == S_OK)
      c_msg(INFO_CHANGED, TRUE);
    else
      c_msg(ERR_FAILURE, TRUE);
    pressreturn();
  }
  return FULLUPDATE;
}

BoardInfo()
{   
  PATH infofile;
  FILE *fp1, *fp2;
  int c;
  extern int lang;

  if (bbs_get_info(infofile) != S_OK) {
    c_msg(ERR_FILE, TRUE);
    pressreturn();
  }
  if ((fp2 = fopen(c_tempfile, "w")) == NULL) {
    c_msg(ERR_FILE, TRUE);
    pressreturn();
  }
  fprintf(fp2, "\t\t%s\n\n", version[lang]);
  if ((fp1 = fopen(infofile, "r")) == NULL) {
    fclose(fp2);
    c_msg(ERR_FILE, TRUE);
    pressreturn();
  }
  while ((c=getc(fp1)) != EOF)
    putc(c, fp2);
  fclose(fp1);
  fclose(fp2);

  clear();
  if (More(c_tempfile, 1)) {
    c_msg(ERR_FAILURE, TRUE);
    unlink(c_tempfile);
    pressreturn();
  }
  unlink(c_tempfile);
  return FULLUPDATE;
}

GnuInfo()
{   
  PATH gnufile;
  clear();
  if (bbs_get_license(gnufile) != S_OK || More(gnufile, 1)) {
    c_msg(ERR_FAILURE, TRUE);
    pressreturn();
  }
  return FULLUPDATE;
}

#define MAXLINELEN 256

CopyMessageBody(dest, src)
char *dest, *src;
{
  int LINELEN;
  char buf[MAXLINELEN], overflow[MAXLINELEN];
  FILE *ofp, *ifp;
  int ch, lastch = 0;
  int i, inbody = 0, newline = 1, overflowed = 0;
  char *space;

  LINELEN = t_columns;

  if ((ofp = fopen(dest, "w")) == NULL) return -1;
  if ((ifp = fopen(src, "r")) == NULL) {
    fclose(ofp);
    return -1;
  }

  while ((ch = fgetc(ifp)) != EOF) {
    if (!inbody) {
      if (ch == '\n' && lastch == '\n') inbody = 1;
      else lastch = ch;
      continue;
    }
    if (newline) {
      newline = 0;
      if (overflowed) {
        memcpy(buf, overflow, LINELEN);
        overflowed = 0;
      }              
      else memset(buf, 0, LINELEN);
      i = strlen(buf);
    }
    buf[i++] = ch;
    if (i > LINELEN-4 && ch != '\n') {
      if ((space = strrchr(buf, ' ')) != NULL) {
        *space++ = '\0';
        memset(overflow, 0, LINELEN);
        strcpy(overflow, space);
        overflowed = 1;        
      }
      fprintf(ofp, "%s\n", buf);
      newline = 1;
    }
    if (ch == '\n') {
      fprintf(ofp, "%s", buf);
      newline = 1;
    }
  }
  fclose(ifp);
  fclose(ofp);
}

AnnotateMessageBody(dest, src)
char *dest, *src;
{
  int LINELEN;
  char buf[MAXLINELEN], overflow[MAXLINELEN];
  FILE *ofp, *ifp;
  int ch, lastch = 0;
  int i, inbody = 0, newline = 1, overflowed = 0;
  char *space;

  LINELEN = t_columns;

  if ((ofp = fopen(dest, "w")) == NULL) return -1;
  if ((ifp = fopen(src, "r")) == NULL) {
    fclose(ofp);
    return -1;
  }

  while ((ch = fgetc(ifp)) != EOF) {
    if (!inbody) {
      if (ch == '\n' && lastch == '\n') inbody = 1;
      else lastch = ch;
      continue;
    }
    if (newline) {
      fprintf(ofp, "> ");
      newline = 0;
      if (overflowed) {
        memcpy(buf, overflow, LINELEN);
        overflowed = 0;
      }              
      else memset(buf, 0, LINELEN);
      i = strlen(buf);
    }
    buf[i++] = ch;
    if (i > LINELEN-4 && ch != '\n') {
      if ((space = strrchr(buf, ' ')) != NULL) {
        *space++ = '\0';
        memset(overflow, 0, LINELEN);
        strcpy(overflow, space);
        overflowed = 1;        
      }
      fprintf(ofp, "%s\n", buf);
      newline = 1;
    }
    if (ch == '\n') {
      fprintf(ofp, "%s", buf);
      newline = 1;
    }
  }
  fclose(ifp);
  fclose(ofp);
}

filecopy(dest, src)
char *dest, *src;
{
  int ifd, ofd, rc = 0, cc;
  char buf[1024];
  if ((ofd = open(dest, O_WRONLY | O_CREAT, 0600)) == -1)
    return -1;
  if ((ifd = open(src, O_RDONLY)) == -1)
    {
      close(ofd);
      return -2;
    }
  while (cc = read(ifd, buf, sizeof(buf)))
    if (write(ofd, buf, cc) != cc) {
      rc = -3;
      break;
    }
  close(ofd);
  close(ifd);
  return rc;
}

#if REMOTE_CLIENT
SaveFile(fname)
char *fname;
{
  char savename[PATHLEN];
  clear();
  getdata(0, 0, "Save to file: ", savename, sizeof(savename), DOECHO, 0);
  if (savename[0] == '\0')
    {
      c_msg(INFO_CANCELLED, TRUE);
    }
  else
    switch(filecopy(savename, fname))
      {
      case -3:
      case -2:
      case -1:
	c_msg(ERR_FILE, TRUE);
	break;
      case 0:
	c_msg(INFO_SAVED, TRUE);
      }
}
#endif
